/*****************************************************************************
 *
 * This program uses the GLUI library of Paul Rademacher (and the GLUT)
 * and the original particle model ideas of Richard Benson and Brian Tischler 
 * to produce an example of a very interactive particle system visualiser
 *
 *****************************************************************************/

#include <string.h>
#include <math.h>  

#if _WIN32    // windows

#include <windows.h>
#include <time.h>
#pragma warning( disable : 4244 4305 4267 4101 )
#include "GL\glut.h"
#include "GL\glui.h"

#else   // mac

#include <sys/time.h>
#include <stdlib.h>
#include <opengl/gl.h>
#include <opengl/glu.h>
#include <GLUT/GLUT.h>
#include "gl/glui.h"
#define TRUE 1
#define FALSE 0
#define BOOL int
#define TCHAR char
#define UINT unsigned int
#define LPSTR char *
#define WORD unsigned short
#define DWORD unsigned long
#define LONG long
#define BYTE unsigned char

typedef struct tagBITMAPFILEHEADER {
	WORD bfType; 
	DWORD bfSize; 
	WORD bfReserved1; 
	WORD bfReserved2; 
	DWORD bfOffBits; 
} BITMAPFILEHEADER; 

typedef struct tagBITMAPINFOHEADER { 
	DWORD biSize; 
	LONG biWidth; 
	LONG biHeight; 
	WORD biPlanes; 
	WORD biBitCount;
	DWORD biCompression; 
	DWORD biSizeImage; 
	LONG biXPelsPerMeter; 
	LONG biYPelsPerMeter; 
	DWORD biClrUsed; 
	DWORD biClrImportant; 
} BITMAPINFOHEADER; 

typedef struct tagRGBQUAD {
        BYTE    rgbBlue;
        BYTE    rgbGreen;
        BYTE    rgbRed;
        BYTE    rgbReserved;
} RGBQUAD;

typedef struct tagBITMAPINFO {
    BITMAPINFOHEADER    bmiHeader;
    RGBQUAD             bmiColors[1];
} BITMAPINFO;

float timeGetTime(void){
 static timeval tv;
 gettimeofday(&tv,NULL);
 double t1=(tv.tv_sec) - 1364335233.0;
 double t2=(tv.tv_usec);
 return (t1*1000.0 + t2/1000.0);
}
#endif

#include "Particle.h"
#include "ParticleSystem.h"


int InitParticleSystem(void);
BOOL LoadBMP(char* szFileName);
void SetViewpoint(void);
void DoPresetFunction(void);

// This is the main particle system 
aParticleSystem ParticleSystemInstance; // particle system instance
aParticleSystem* pPS;        // pointer to particle system instance

int iTimeUp = 0;             // flag for particle burst (e.g. fireworks)
int iPresetKey = 1;          // which preset to use
int iBurstOn = 10;           // burst counter (when a burst occurs this counter is started and then counts down, when 0 the burst is stopped
int bIsBurstEnable = 0;      // burst mode is enabled 
float fTime;                 // time of the current frame
float fStartBurstTime;       // burst start time
int bIsCamFollowing = FALSE; // is the camera following the source

// camera settings
float fCamRotX,fCamRotY;     // rotations to apply to objects to get the view from the camera's current location
vector vCamPos;              // camera position - calculated from radius, yaw and pitch
float fCameraRadius;         // distance from viewpoint
float fCameraYaw;            // camera Yaw angle
float fCameraPitch;          // camera Pitch angle

GLuint tList[7];             // OpenGL texture list identifier

// calculate the viewing point trnsformation 
void SetViewpoint(int id){ 
 if ( id == 1 ) fCameraYaw  += 0.05f; // left  - these are set via the arrow keys
 if ( id == 2 ) fCameraYaw  -= 0.05f; // right
 if ( id == 3 ) fCameraPitch -= 0.05f; // up 
 if ( id == 4 ) fCameraPitch += 0.05f; // down 
 if ( id == 5 ) fCameraRadius -= 2.0f;  // A
 if ( id == 6 ) fCameraRadius += 2.0f;  // Z
 //make a little spin for effect when following moving particles
 if(bIsCamFollowing && id > 0) fCameraYaw+=0.005f;
 // wrap Yaw rotation at edges for continous movement
 if ( fCameraYaw < 0.0f )fCameraYaw  = 6.28f;
 else if ( fCameraYaw > 6.28f ) fCameraYaw = 0.0f;
 Clamp( fCameraPitch, -1.57f, 1.57f );
 Clamp( fCameraRadius, 1.0f, 200.0f );
 // spherical coordinate rotation
 vCamPos.x = (float)(cos(fCameraYaw) * cos(fCameraPitch) * fCameraRadius);
 vCamPos.z = (float)(sin(fCameraYaw) * cos(fCameraPitch) * fCameraRadius);
 vCamPos.y = (float)(sin(fCameraPitch) * fCameraRadius);
 fCamRotY = fCameraYaw*360.0f/-6.28f+90.0f;
 fCamRotX = fCameraPitch*360.0f/-6.28f;
}

int InitParticleSystem(void){ // initialise the particle system
 if ( !pPS ){  return (FALSE); }
    //---- Camera Settings ---- 
   fCameraPitch  =  0.15f; 
   fCameraYaw   =  1.56f; 
   fCameraRadius =  37.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.48f; 
   pPS->vColorStart.g = 0.42f; 
   pPS->vColorStart.b = 0.10f; 
   pPS->vColorVar.r = 0.15f; 
   pPS->vColorVar.g = 0.15f; 
   pPS->vColorVar.b = 0.15f; 
   pPS->vColorEnd.r = 0.04f; 
   pPS->vColorEnd.g = 0.22f; 
   pPS->vColorEnd.b = 0.81f; 
   pPS->fAlphaStart = 0.92f; 
   pPS->fAlphaVar   = 0.14f; 
   pPS->fAlphaEnd   = 0.14f; 
   pPS->fSizeStart  = 1.04f; 
   pPS->fSizeVar    = 0.04f; 
   pPS->fSizeEnd    = 1.08f; 
   pPS->fSpeed      = 3.00f; 
   pPS->fSpeedVar   = 12.00f; 
   pPS->fTheta      = 0.20f; 
   pPS->fLifeTime   = 0.60f; 
   pPS->fLifeVar    = 0.10f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.00f; 
   pPS->vGravityStart.y  = 0.00f; 
   pPS->vGravityEnd.y   = 0.00f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 20; 
   pPS->fParticlesPerSec = 20; 
   pPS->bIsMoving     = 0; 
   pPS->bIsColliding  = 1; 
   bIsCamFollowing  = 0; 
   bIsBurstEnable  = 0; 
   pPS->fBoing     = 0.30f; 
   pPS->iParticleTexture = 1; 
 //Initialize the camera points
 vCamPos.x = (float)(cos(fCameraYaw) * cos(fCameraPitch) * fCameraRadius);
 vCamPos.z = (float)(sin(fCameraYaw) * cos(fCameraPitch) * fCameraRadius);
 vCamPos.y = (float)(sin(fCameraPitch) * fCameraRadius);
 fCamRotY = fCameraYaw*360.0f/-6.28f+90.0f;
 fCamRotX = fCameraPitch*360.0f/-6.28f;
 //So the system knows what time it started.
 pPS->fTimeLastUpdate = (float)(timeGetTime())*0.001f;
 fStartBurstTime = (float)(timeGetTime())*0.001f;
 return (TRUE);
}

void DoPresetFunction(void){ // set the example preset positions for some example particle systems
 switch (iPresetKey){
  case 1:
   //---- Camera Settings ---- 
   fCameraPitch  =  0.15f; 
   fCameraYaw   =  1.58f; 
   fCameraRadius =  53.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.48f; 
   pPS->vColorStart.g = 0.17f; 
   pPS->vColorStart.b = 0.10f; 
   pPS->vColorVar.r = 0.15f; 
   pPS->vColorVar.g = 0.15f; 
   pPS->vColorVar.b = 0.15f; 
   pPS->vColorEnd.r = 0.04f; 
   pPS->vColorEnd.g = 0.60f; 
   pPS->vColorEnd.b = 0.81f; 
   pPS->fAlphaStart = 0.46f; 
   pPS->fAlphaVar   = 0.14f; 
   pPS->fAlphaEnd   = 0.14f; 
   pPS->fSizeStart  = 0.26f; 
   pPS->fSizeVar    = 0.04f; 
   pPS->fSizeEnd    = 1.00f; 
   pPS->fSpeed      = 1.50f; 
   pPS->fSpeedVar   = 5.00f; 
   pPS->fTheta      = 0.96f; 
   pPS->fLifeTime   = 0.90f; 
   pPS->fLifeVar    = 1.10f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.00f; 
   pPS->vGravityStart.y  = 0.08f; 
   pPS->vGravityEnd.y   = 0.00f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 273; 
   pPS->fParticlesPerSec = 273; 
   pPS->bIsMoving     = 1; 
   pPS->bIsColliding  = 1; 
   pPS->fBoing     = 0.30f; 
   pPS->iParticleTexture = 1; 
   bIsCamFollowing = FALSE;
   bIsBurstEnable = FALSE;
   break;
   
  case 2:
   //---- Camera Settings ---- 
   fCameraPitch  =  -0.08f; 
   fCameraYaw   =  0.16f; 
   fCameraRadius =  41.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.28f; 
   pPS->vColorStart.g = 0.29f; 
   pPS->vColorStart.b = 0.30f; 
   pPS->vColorVar.r = 0.85f; 
   pPS->vColorVar.g = 0.85f; 
   pPS->vColorVar.b = 0.85f; 
   pPS->vColorEnd.r = 0.28f; 
   pPS->vColorEnd.g = 0.29f; 
   pPS->vColorEnd.b = 0.30f; 
   pPS->fAlphaStart = 0.96f; 
   pPS->fAlphaVar   = 0.01f; 
   pPS->fAlphaEnd   = 0.97f; 
   pPS->fSizeStart  = 0.12f; 
   pPS->fSizeVar    = 1.12f; 
   pPS->fSizeEnd    = 0.12f; 
   pPS->fSpeed      = 3.25f; 
   pPS->fSpeedVar   = 13.00f; 
   pPS->fTheta      = 0.16f; 
   pPS->fLifeTime   = 10.20f; 
   pPS->fLifeVar    = 0.75f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.04f; 
   pPS->vGravityStart.y  = 0.07f; 
   pPS->vGravityEnd.y   = -0.48f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 6; 
   pPS->fParticlesPerSec = 6; 
   pPS->bIsMoving     = 1; 
   pPS->bIsColliding  = 1; 
   bIsCamFollowing  = 0; 
   bIsBurstEnable  = 0; 
   pPS->fBoing     = 0.75f; 
   pPS->iParticleTexture = 2; 
   break;

  case 3:
   //---- Camera Settings ---- 
   fCameraPitch  =  0.22f; 
   fCameraYaw   =  0.25f; 
   fCameraRadius =  57.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.06f; 
   pPS->vColorStart.g = 0.05f; 
   pPS->vColorStart.b = 0.18f; 
   pPS->vColorVar.r = 0.13f; 
   pPS->vColorVar.g = 0.13f; 
   pPS->vColorVar.b = 0.13f; 
   pPS->vColorEnd.r = 0.52f; 
   pPS->vColorEnd.g = 0.29f; 
   pPS->vColorEnd.b = 0.04f; 
   pPS->fAlphaStart = 0.96f; 
   pPS->fAlphaVar   = 0.23f; 
   pPS->fAlphaEnd   = 0.03f; 
   pPS->fSizeStart  = 0.90f; 
   pPS->fSizeVar    = 0.46f; 
   pPS->fSizeEnd    = 1.98f; 
   pPS->fSpeed      = 9.75f; 
   pPS->fSpeedVar   = 13.00f; 
   pPS->fTheta      = 0.14f; 
   pPS->fLifeTime   = 6.30f; 
   pPS->fLifeVar    = 0.75f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.00f; 
   pPS->vGravityStart.y  = -0.08f; 
   pPS->vGravityEnd.y   = 0.12f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 140; 
   pPS->fParticlesPerSec = 140; 
   pPS->bIsMoving     = 0; 
   pPS->bIsColliding  = 1; 
   bIsCamFollowing  = 0; 
   bIsBurstEnable  = 0; 
   pPS->fBoing     = 0.29f; 
   pPS->iParticleTexture = 1; 
   break;
  
  case 4:
    //---- Camera Settings ---- 
   fCameraPitch  =  1.57f; 
   fCameraYaw   =  4.30f; 
   fCameraRadius =  58.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.81f; 
   pPS->vColorStart.g = 0.82f; 
   pPS->vColorStart.b = 0.83f; 
   pPS->vColorVar.r = 0.24f; 
   pPS->vColorVar.g = 0.24f; 
   pPS->vColorVar.b = 0.24f; 
   pPS->vColorEnd.r = 0.82f; 
   pPS->vColorEnd.g = 0.83f; 
   pPS->vColorEnd.b = 0.83f; 
   pPS->fAlphaStart = 0.02f; 
   pPS->fAlphaVar   = 0.14f; 
   pPS->fAlphaEnd   = 0.66f; 
   pPS->fSizeStart  = 0.10f; 
   pPS->fSizeVar    = 0.28f; 
   pPS->fSizeEnd    = 1.00f; 
   pPS->fSpeed      = 14.25f; 
   pPS->fSpeedVar   = 23.75f; 
   pPS->fTheta      = 0.79f; 
   pPS->fLifeTime   = 2.25f; 
   pPS->fLifeVar    = 0.40f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.00f; 
   pPS->vGravityStart.y  = 0.04f; 
   pPS->vGravityEnd.y   = 0.44f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 80; 
   pPS->fParticlesPerSec = 80; 
   pPS->bIsMoving     = 0; 
   pPS->bIsColliding  = 0; 
   bIsCamFollowing  = 0; 
   pPS->fBoing     = 0.14f; 
   pPS->iParticleTexture = 5; 
   bIsBurstEnable = FALSE;
   break;

  case 5:
   //---- Camera Settings ---- 
   fCameraPitch  =  1.57f; 
   fCameraYaw   =  4.30f; 
   fCameraRadius =  200.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.52f; 
   pPS->vColorStart.g = 0.34f; 
   pPS->vColorStart.b = 0.53f; 
   pPS->vColorVar.r = 0.96f; 
   pPS->vColorVar.g = 0.96f; 
   pPS->vColorVar.b = 0.96f; 
   pPS->vColorEnd.r = 0.52f; 
   pPS->vColorEnd.g = 0.34f; 
   pPS->vColorEnd.b = 0.52f; 
   pPS->fAlphaStart = 0.33f; 
   pPS->fAlphaVar   = 0.14f; 
   pPS->fAlphaEnd   = 0.99f; 
   pPS->fSizeStart  = 0.32f; 
   pPS->fSizeVar    = 0.36f; 
   pPS->fSizeEnd    = 1.88f; 
   pPS->fSpeed      = 3.50f; 
   pPS->fSpeedVar   = 23.75f; 
   pPS->fTheta      = 0.025f; 
   pPS->fLifeTime   = 10.50f; 
   pPS->fLifeVar    = 0.15f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.00f; 
   pPS->vGravityStart.y  = 0.12f; 
   pPS->vGravityEnd.y   = -0.12f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 35; 
   pPS->fParticlesPerSec = 35; 
   pPS->bIsMoving     = 1; 
   pPS->bIsColliding  = 0; 
   bIsCamFollowing  = 0; 
   pPS->fBoing     = 0.14f; 
   pPS->iParticleTexture = 3; 
   bIsBurstEnable = FALSE;
   break;
   
  case 6:
   //---- Camera Settings ---- 
   fCameraPitch  =  1.57f; 
   fCameraYaw   =  5.88f; 
   fCameraRadius =  63.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.44f; 
   pPS->vColorStart.g = 0.38f; 
   pPS->vColorStart.b = 0.76f; 
   pPS->vColorVar.r = 0.14f; 
   pPS->vColorVar.g = 0.14f; 
   pPS->vColorVar.b = 0.14f; 
   pPS->vColorEnd.r = 0.51f; 
   pPS->vColorEnd.g = 0.68f; 
   pPS->vColorEnd.b = 0.96f; 
   pPS->fAlphaStart = 0.02f; 
   pPS->fAlphaVar   = 0.01f; 
   pPS->fAlphaEnd   = 0.94f; 
   pPS->fSizeStart  = 0.10f; 
   pPS->fSizeVar    = 0.10f; 
   pPS->fSizeEnd    = 0.42f; 
   pPS->fSpeed      = 12.75f; 
   pPS->fSpeedVar   = 0.00f; 
   pPS->fTheta      = 0.30f; 
   pPS->fLifeTime   = 2.85f; 
   pPS->fLifeVar    = 0.40f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.00f; 
   pPS->vGravityStart.y  = 0.04f; 
   pPS->vGravityEnd.y   = 0.44f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 125; 
   pPS->fParticlesPerSec = 125; 
   pPS->bIsMoving     = 1; 
   pPS->bIsColliding  = 0; 
   bIsCamFollowing  = 0; 
   pPS->fBoing     = 0.69f; 
   pPS->iParticleTexture = 2; 
   bIsBurstEnable = FALSE;
   break;
  
  case 7:
   
   //---- Camera Settings ---- 
   fCameraPitch  =  -1.35f; 
   fCameraYaw   =  1.58f; 
   fCameraRadius =  101.00f; 
   //---- Particle System Settings ---- 
   pPS->vColorStart.r = 0.70f; 
   pPS->vColorStart.g = 0.94f; 
   pPS->vColorStart.b = 0.94f; 
   pPS->vColorVar.r = 0.01f; 
   pPS->vColorVar.g = 0.01f; 
   pPS->vColorVar.b = 0.01f; 
   pPS->vColorEnd.r = 0.21f; 
   pPS->vColorEnd.g = 0.04f; 
   pPS->vColorEnd.b = 0.67f; 
   pPS->fAlphaStart = 0.97f; 
   pPS->fAlphaVar   = 0.01f; 
   pPS->fAlphaEnd   = 0.98f; 
   pPS->fSizeStart  = 1.20f; 
   pPS->fSizeVar    = 0.04f; 
   pPS->fSizeEnd    = 1.40f; 
   pPS->fSpeed      = 24.25f; 
   pPS->fSpeedVar   = 0.00f; 
   pPS->fTheta      = 0.55f; 
   pPS->fLifeTime   = 1.65f; 
   pPS->fLifeVar    = 2.10f; 
   pPS->vGravityStart.x  = 0.00f; 
   pPS->vGravityEnd.x   = 0.00f; 
   pPS->vGravityStart.y  = 0.28f; 
   pPS->vGravityEnd.y   = -0.20f; 
   pPS->vGravityStart.z  = 0.00f; 
   pPS->vGravityEnd.z   = 0.00f; 
   pPS->fGravityVar   = 0.00f; 
   pPS->uParticlesPerSec = 940; 
   pPS->fParticlesPerSec = 940; 
   pPS->bIsMoving     = 1; 
   pPS->bIsColliding  = 0; 
   bIsCamFollowing  = 0; 
   pPS->fBoing     = 0.30f; 

   pPS->iParticleTexture = 3; 
   bIsBurstEnable = TRUE;
   break;
  }
  SetViewpoint(0); // update camera position only
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////

void LoadTextures(void){  // load the texture images 
 glGenTextures(7, tList);
 glBindTexture(GL_TEXTURE_2D, tList[1]);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 LoadBMP("star.bmp");
 glBindTexture(GL_TEXTURE_2D, tList[2]);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 LoadBMP("bubble.bmp");
 glBindTexture(GL_TEXTURE_2D, tList[3]);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 LoadBMP("spark.bmp");
 glBindTexture(GL_TEXTURE_2D, tList[4]);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 LoadBMP("circle.bmp");
 glBindTexture(GL_TEXTURE_2D, tList[5]);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 LoadBMP("cloud.bmp");
}

// utility functions to load the particle image maps from a Windows BITMAP
BOOL LoadBMP(char* szFileName) {
 FILE *hFileHandle;
 BITMAPINFO *pBitmapInfo = NULL;
 unsigned long lInfoSize = 0;
 unsigned long lBitSize = 0;
 int nTextureWidth;
 int nTextureHeight;
 BYTE *pBitmapData;

 hFileHandle=fopen(szFileName,"rb");
 if(hFileHandle == NULL)return FALSE; 
 BITMAPFILEHEADER bitmapHeader;
 DWORD dwBytes;
 fread(&bitmapHeader.bfType,2,1,hFileHandle); // numbers hard coded for use on Mac // must compile with 32 bit library on Mac
 fread(&bitmapHeader.bfSize,4,1,hFileHandle);
 fread(&bitmapHeader.bfReserved1,2,1,hFileHandle);
 fread(&bitmapHeader.bfReserved1,2,1,hFileHandle);
 fread(&bitmapHeader.bfOffBits,4,1,hFileHandle);
 if(bitmapHeader.bfType != 'MB')return FALSE;
 lInfoSize = bitmapHeader.bfOffBits - 14;
//printf("Reading [%s] Offs=%d\n",szFileName,bitmapHeader.bfOffBits);
 pBitmapInfo = (BITMAPINFO *) new BYTE[lInfoSize];
 dwBytes=fread(pBitmapInfo,1,lInfoSize,hFileHandle);
 if(dwBytes != lInfoSize)   return FALSE;
 nTextureWidth = pBitmapInfo->bmiHeader.biWidth;
 nTextureHeight = pBitmapInfo->bmiHeader.biHeight;
 lBitSize = pBitmapInfo->bmiHeader.biSizeImage;
 if(lBitSize == 0)
   lBitSize = (nTextureWidth *
               pBitmapInfo->bmiHeader.biBitCount + 7) / 8 *
       abs(nTextureHeight);
 pBitmapData = new BYTE[lBitSize];
//printf("Read data %d\n",lBitSize);
 dwBytes=fread(pBitmapData,1,lBitSize,hFileHandle);
 if(lBitSize != dwBytes)   {
   if(pBitmapData)delete [] (BYTE *) pBitmapData;
   pBitmapData = NULL;
   return FALSE;
 }
 fclose(hFileHandle);
 if(pBitmapInfo != NULL) delete [] (BYTE *)pBitmapInfo;
 glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
 glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
 glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
 glTexImage2D(GL_TEXTURE_2D, 0, 3, nTextureWidth, nTextureHeight, 0,
                 GL_BGR_EXT, GL_UNSIGNED_BYTE, pBitmapData);
 if(pBitmapData)  delete [] (BYTE *) pBitmapData;
 return TRUE;
}

//////////////////////////////////////////////////////
////////////////         GLUI and GLUT code 
//////////////////////////////////////////////////////

int   main_window;  // GLUI output window 
char *string_list[] = { "Star", "Bubble", "Spark", "Circle", "Cloud" }; // string identfiers for texture images 
int   curr_string = 0;


GLUI   *glui;  /* Pointer to the GLUI window */

/* IDs for GLUI controls */
#define ALPHA_START_ID         200
#define ALPHA_END_ID           201
#define RED_START_ID           202
#define RED_END_ID             203
#define GREEN_START_ID         204
#define GREEN_END_ID           205
#define BLUE_START_ID          206
#define BLUE_END_ID            207

#define SIZE_START_ID          300
#define SIZE_END_ID            301
#define LIFE_ID                302
#define EMIT_ID                304
#define SPEED_ID               306

#define THETA_ID               400
#define BOING_ID               401

#define GRAVITY_X_START_ID     402
#define GRAVITY_Y_START_ID     403
#define GRAVITY_Z_START_ID     404
#define GRAVITY_X_END_ID       405
#define GRAVITY_Y_END_ID       406
#define GRAVITY_Z_END_ID       407

#define BURST_ID               500
#define GROUND_ID              501
#define MOVING_ID              502
#define CAMERA_ID              503

#define IMAGE_ID               600

#define VARIABILITY_ALPHA_ID   700
#define VARIABILITY_COLOUR_ID  701
#define VARIABILITY_SIZE_ID    702
#define VARIABILITY_LIFE_ID    703
#define VARIABILITY_SPEED_ID   704


void control_cb( int control ){   // GLUI control callback function
  if(control == VARIABILITY_COLOUR_ID){ // update the other color components
    pPS->vColorVar.g = pPS->vColorVar.b = pPS->vColorVar.r;
  }
  else if(control == IMAGE_ID){
    pPS->iParticleTexture=curr_string+1;   // assign the particle system's image ID
  }
  else if (control == 901){iPresetKey = 1;  DoPresetFunction();}
  else if (control == 902){iPresetKey = 2;  DoPresetFunction();}
  else if (control == 903){iPresetKey = 3;  DoPresetFunction();}
  else if (control == 904){iPresetKey = 4;  DoPresetFunction();}
  else if (control == 905){iPresetKey = 5;  DoPresetFunction();}
  else if (control == 906){iPresetKey = 6;  DoPresetFunction();}
  else if (control == 907){iPresetKey = 7;  DoPresetFunction();}
}

void SpecialKey(int key, int x, int y){ // arrow keys etc.
    switch (key) {
     case GLUT_KEY_UP:
       SetViewpoint(3); // change camera viewpoint
       break;
     case GLUT_KEY_DOWN:
       SetViewpoint(4);
       break;
     case GLUT_KEY_LEFT:
       SetViewpoint(1);
       break;
     case GLUT_KEY_RIGHT:
       SetViewpoint(2);
       break;
     // next batch sets up some defaults F1 to F7 function keys
     case GLUT_KEY_F1: iPresetKey = 1;  DoPresetFunction();  break;  
     case GLUT_KEY_F2: iPresetKey = 2;  DoPresetFunction();  break;
     case GLUT_KEY_F3: iPresetKey = 3;  DoPresetFunction();  break;
     case GLUT_KEY_F4: iPresetKey = 4;  DoPresetFunction();  break;
     case GLUT_KEY_F5: iPresetKey = 5;  DoPresetFunction();  break;
     case GLUT_KEY_F6: iPresetKey = 6;  DoPresetFunction();  break;
     case GLUT_KEY_F7: iPresetKey = 7;  DoPresetFunction();  break;
    }
}

void myGlutKeyboard(unsigned char Key, int x, int y){
  switch(Key) {
  case 27:    //  "ESc"
  case 'q':
    exit(0);
    break;
  case 'A':  // zoom in 
  case 'a':
    SetViewpoint(5); // update the camera position
    break;
  case 'Z':  // zoom out
  case 'z':
    SetViewpoint(6);
    break;
  };
  glutPostRedisplay();
}

// used for mouse positioning - the mouse is us used to change the viewpoint and zoom in and out
static int b_mb=0;
static int screen_x,screen_y;
static int b_mb2=0;

void myGlutMouse(int button, int button_state, int x, int y ){ // mouse button
  if (button == GLUT_LEFT_BUTTON)   {
      if (button_state == GLUT_UP)      {
        b_mb=0;
      }
      else {
        b_mb=1;
        screen_x=x; screen_y=y;
      }
  }
  if (button == GLUT_RIGHT_BUTTON)   {
      if (button_state == GLUT_UP) {
        b_mb2=0;
        screen_x=x; screen_y=y;
      }
      else {
        b_mb2=1;
        screen_x=x; screen_y=y;
      }
  }
  if (button == GLUT_MIDDLE_BUTTON)   {
    if (button_state == GLUT_DOWN)      {
    }
  } 
}

void myGlutMotion(int x, int y ){ // mouse move with button down - use to change viewpoint
  int dx,dy;
  if(b_mb){
    dx=x-screen_x; dy=y-screen_y;
    screen_x=x; screen_y=y;
    fCameraYaw  += (float)dx*0.005f; // left
    fCameraPitch += (float)dy*0.005f; // up 
    SetViewpoint(0);  // update the view point
    glutPostRedisplay();
  }
  if(b_mb2){
    dx=x-screen_x; dy=y-screen_y;
    screen_x=x; screen_y=y;
    fCameraRadius += 0.1*(float)dy;
    SetViewpoint(0);
    glutPostRedisplay();
  }
}


void myGlutReshape( int x, int y ){ // called when window changes shape 
 int tx, ty, tw, th;
 // the GLUI pannels take up space - get the drawing window's dimensions 
 GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th ); 
 glViewport( tx, ty, tw, th );
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluPerspective(45.0f,(float)tw/(float)th,0.1f,1000.0f);
 glMatrixMode(GL_MODELVIEW);
 glLoadIdentity();
 glutPostRedisplay();
}

void myGlutIdle( void ){ // use this function to control bursts of particles
  if ( glutGetWindow() != main_window )glutSetWindow(main_window);  
  if(iTimeUp)  { // if burst timer has fired
   iTimeUp = 0;
   if(bIsBurstEnable)  {
    if (iBurstOn){
     pPS->bIsSuppressed = TRUE;
     iBurstOn --;
    }
    else{
     pPS->bIsSuppressed = FALSE;
     iBurstOn = rand()%20+7;
    }
   }
   else pPS->bIsSuppressed = FALSE;
  }
  glutPostRedisplay();
}

void myGlutDisplay( void ){  // render the Particles with OpenGL
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  
 glLoadIdentity();
 glPushMatrix();
 // set up the viewpoint 
 if(bIsCamFollowing) gluLookAt( // camera is moving with emitter
      pPS->vLocation.x+vCamPos.x, pPS->vLocation.y*2.0f+vCamPos.y, pPS->vLocation.z+vCamPos.z,
      pPS->vLocation.x, pPS->vLocation.y, pPS->vLocation.z,
      0.0f,1.0f,0.0f); // up 
 else gluLookAt( // static camera 
      vCamPos.x, vCamPos.y, vCamPos.z, 
      0.0f,0.0f,0.0f,
      0.0f,1.0f,0.0f); // up
 fTime = (float)(timeGetTime())*0.001f; // get the time (used to update the particle positions in real-time
 if(fTime > fStartBurstTime + 0.1){ // 100ms later
   fStartBurstTime=fTime;
   iTimeUp=1;   // flag that we want a burst to start (handled in the idel function)
 }
 UpdateParticleSystem(pPS,fTime);   // update the particlesystem and the particles positions for current time
 glBindTexture(GL_TEXTURE_2D, tList[pPS->iParticleTexture]);
 DrawParticleSystemParticles(pPS,fCamRotX,fCamRotY);  // draw the particles in particle system "pPS" from this point of view
 glPopMatrix();
 glutSwapBuffers(); 
}

void InitGL(void){
 glEnable(GL_TEXTURE_2D);     // Enable Texture Mapping
 glClearColor(0.0f, 0.0f, 0.0f, 0.0f);  // This Will Clear The Background Color To Black
 glClearDepth(1.0);       // Enables Clearing Of The Depth Buffer
 glShadeModel(GL_SMOOTH);     // Enables Smooth Color Shading
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();       // Reset The Projection Matrix
// gluPerspective(45.0f,1.3333f,0.1f,1000.0f); // Calculate The Aspect Ratio Of The Window
 glMatrixMode(GL_MODELVIEW);
 glBlendFunc(GL_SRC_ALPHA,GL_ONE);   // Set The Blending Function For Translucency
 glEnable(GL_BLEND);
 glEnable(GL_CULL_FACE);
 // glEnable(GL_DEPTH_TEST); is NOT used for particles
}

int main(int argc, char* argv[]){
  // pPS is the global pointer that is used for ALL particle system access
  pPS = &ParticleSystemInstance;    
  
  srand( (unsigned)time( NULL ) );
  // set up the GLUT 
  glutInit(&argc, argv);  
  glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
  glutInitWindowPosition(2,4);
  glutInitWindowSize(1240,960);
  main_window = glutCreateWindow( "Particle Foundry" );

  InitParticleSystem();  // set fefault values for the particle systems parameters
  LoadTextures(); // load the texture images
  InitGL();

  /* Create the side subwindow */
  glui = GLUI_Master.create_glui_subwindow( main_window, 
					    GLUI_SUBWINDOW_RIGHT );
  GLUI_Scrollbar *sb;  
  /* Add the colour controls */ 
  GLUI_Rollout *roll_colours = new GLUI_Rollout(glui, "Colours", true );
  GLUI_Panel *alpha = new GLUI_Panel( roll_colours, "Alpha Start/End" );
  sb = new GLUI_Scrollbar(alpha, "",GLUI_SCROLL_HORIZONTAL,&(pPS->fAlphaStart),ALPHA_START_ID,control_cb);
  sb->set_float_limits(MIN_ALPHA,MAX_ALPHA);
  sb = new GLUI_Scrollbar(alpha, "",GLUI_SCROLL_HORIZONTAL,&(pPS->fAlphaEnd),ALPHA_END_ID,control_cb);
  sb->set_float_limits(MIN_ALPHA,MAX_ALPHA);
  GLUI_Panel *red   = new GLUI_Panel( roll_colours, "Red Start/End" );
  sb = new GLUI_Scrollbar(red, "",GLUI_SCROLL_HORIZONTAL,&(pPS->vColorStart.r),RED_START_ID,control_cb);
  sb->set_float_limits(0.04,0.96);
  sb = new GLUI_Scrollbar(red, "",GLUI_SCROLL_HORIZONTAL,&(pPS->vColorStart.r),RED_END_ID,control_cb);
  sb->set_float_limits(0.04,0.96);
  GLUI_Panel *green = new GLUI_Panel( roll_colours, "Green Start/End" );
  sb = new GLUI_Scrollbar(green, "",GLUI_SCROLL_HORIZONTAL,&(pPS->vColorStart.g),GREEN_START_ID,control_cb);
  sb->set_float_limits(0.04,0.96);
  sb = new GLUI_Scrollbar(green, "",GLUI_SCROLL_HORIZONTAL,&(pPS->vColorStart.g),GREEN_END_ID,control_cb);
  sb->set_float_limits(0.04,0.96);
  GLUI_Panel *blue  = new GLUI_Panel( roll_colours, "Blue Start/End" );
  sb = new GLUI_Scrollbar(blue, "",GLUI_SCROLL_HORIZONTAL,&(pPS->vColorStart.b),BLUE_START_ID,control_cb);
  sb->set_float_limits(0.04,0.96);
  sb = new GLUI_Scrollbar(blue, "",GLUI_SCROLL_HORIZONTAL,&(pPS->vColorStart.b),BLUE_END_ID,control_cb);
  sb->set_float_limits(0.04,0.96);

  /* Add the settings controls */
  GLUI_Rollout *roll_settings = new GLUI_Rollout(glui, "Settings", true );
  GLUI_Panel *size = new GLUI_Panel( roll_settings, "Size Start/End" );
  sb = new GLUI_Scrollbar(size,"Start",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fSizeStart),SIZE_START_ID,control_cb);
  sb->set_float_limits(MIN_SIZE,MAX_SIZE);
  sb = new GLUI_Scrollbar(size,"End",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fSizeEnd),SIZE_END_ID,control_cb);
  sb->set_float_limits(MIN_SIZE,MAX_SIZE);
  GLUI_Panel *les   = new GLUI_Panel( roll_settings, "Life/Number/Speed" );
  new GLUI_StaticText(les, "Lifetime" );
  sb = new GLUI_Scrollbar(les,"",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fLifeTime),LIFE_ID,control_cb);
  sb->set_float_limits(MIN_LIFETIME,MAX_LIFETIME);
  new GLUI_StaticText(les, "Number/sec" );
  sb = new GLUI_Scrollbar(les,"",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fParticlesPerSec),LIFE_ID,control_cb);
  sb->set_float_limits(MIN_EMISSION,MAX_EMISSION);
  new GLUI_StaticText(les, "Speed" );
  sb = new GLUI_Scrollbar(les,"",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fSpeed),SPEED_ID,control_cb);
  sb->set_float_limits(MIN_SPEED,MAX_SPEED);
  new GLUI_StaticText(les, "Angular dispersion" );
  sb = new GLUI_Scrollbar(les,"",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fTheta),THETA_ID,control_cb);
  sb->set_float_limits(0.02,1.0);
  new GLUI_StaticText(les, "Bounce" );
  sb = new GLUI_Scrollbar(les,"",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fBoing),BOING_ID,control_cb);
  sb->set_float_limits(0.1,1.0);

  /* Add the options controls */
  GLUI_Rollout *options = new GLUI_Rollout(glui, "Options", true );
  new GLUI_Checkbox( options, "Burst", &(bIsBurstEnable), BURST_ID, control_cb );
  new GLUI_Checkbox( options, "Floor", &(pPS->bIsColliding), GROUND_ID, control_cb );
  glui->add_column_to_panel(options,false);
  new GLUI_Checkbox( options, "Moving", &(pPS->bIsMoving), MOVING_ID, control_cb );
  new GLUI_Checkbox( options, "Camera", &(bIsCamFollowing ), CAMERA_ID, control_cb );

  /* Add the gravity (acceleration) controls */
  GLUI_Rollout *gravity = new GLUI_Rollout(glui, "Gravity", false );
  GLUI_Panel *gravityp;
  gravityp = new GLUI_Panel(gravity, "X Start/End" );
  sb = new GLUI_Scrollbar(gravityp, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->vGravityStart.x),GRAVITY_X_START_ID,control_cb);
  sb->set_float_limits(MIN_GRAVITY,MAX_GRAVITY);
  sb = new GLUI_Scrollbar(gravityp, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->vGravityEnd.x),GRAVITY_X_END_ID,control_cb);
  sb->set_float_limits(MIN_GRAVITY,MAX_GRAVITY);
  gravityp = new GLUI_Panel(gravity, "Y Start/End" );
  sb = new GLUI_Scrollbar(gravityp, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->vGravityStart.y),GRAVITY_Y_START_ID,control_cb);
  sb->set_float_limits(MIN_GRAVITY,MAX_GRAVITY);
  sb = new GLUI_Scrollbar(gravityp, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->vGravityEnd.y),GRAVITY_Y_END_ID,control_cb);
  sb->set_float_limits(MIN_GRAVITY,MAX_GRAVITY);
  gravityp = new GLUI_Panel(gravity, "Z Start/End" );
  sb = new GLUI_Scrollbar(gravityp, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->vGravityStart.z),GRAVITY_Z_START_ID,control_cb);
  sb->set_float_limits(MIN_GRAVITY,MAX_GRAVITY);
  sb = new GLUI_Scrollbar(gravityp, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->vGravityEnd.z),GRAVITY_Z_END_ID,control_cb);
  sb->set_float_limits(MIN_GRAVITY,MAX_GRAVITY);

  /* Add the variability controls */
  GLUI_Rollout *variability = new GLUI_Rollout(glui, "Variability", false );
  new GLUI_StaticText(variability, "Alpha" );
  sb = new GLUI_Scrollbar(variability, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fAlphaVar),VARIABILITY_ALPHA_ID,control_cb);
  sb->set_float_limits(0.01,0.96);
  new GLUI_StaticText(variability, "Colour" );
  sb = new GLUI_Scrollbar(variability, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->vColorVar.r),VARIABILITY_COLOUR_ID,control_cb);
  sb->set_float_limits(0.01,0.96);
  new GLUI_StaticText(variability, "Size" );
  sb = new GLUI_Scrollbar(variability, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fSizeVar),VARIABILITY_SIZE_ID,control_cb);
  sb->set_float_limits(MIN_SIZE,MAX_SIZE);
  new GLUI_StaticText(variability, "Lifetime" );
  sb = new GLUI_Scrollbar(variability, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fLifeVar),VARIABILITY_LIFE_ID,control_cb);
  sb->set_float_limits(MIN_LIFETIME,MAX_LIFETIME);
  new GLUI_StaticText(variability, "Speed" );
  sb = new GLUI_Scrollbar(variability, "",GLUI_SCROLL_HORIZONTAL,
                           &(pPS->fSpeedVar),VARIABILITY_SPEED_ID,control_cb);
  sb->set_float_limits(MIN_SPEED,MAX_SPEED);

  /* Add Image listbox */
  new GLUI_StaticText( glui, "" );
  GLUI_Listbox *list = new GLUI_Listbox( glui, "Image:", &curr_string, IMAGE_ID, control_cb );
  int i;   for( i=0; i<5; i++ )list->add_item( i, string_list[i] );

  // add a presets rollout
  GLUI_Rollout *presets = new GLUI_Rollout(glui, "Presets", false );
  new GLUI_Button(presets, "Preset1",901,control_cb ); // presets
  new GLUI_Button(presets, "Preset2",902,control_cb ); // presets
  new GLUI_Button(presets, "Preset3",903,control_cb ); // presets
  new GLUI_Button(presets, "Preset4",904,control_cb ); // presets
  new GLUI_Button(presets, "Preset5",905,control_cb ); // presets
  new GLUI_Button(presets, "Preset6",906,control_cb ); // presets
  new GLUI_Button(presets, "Preset7",907,control_cb ); // presets

  new GLUI_StaticText( glui, "" );
  new GLUI_Button( glui, "Quit", 0,(GLUI_Update_CB)exit ); // quit button
  /* Link windows to GLUI, and register idle callback */
  glui->set_main_gfx_window( main_window );
  // set up the callback function for the GLUI and GLUT
  glutDisplayFunc( myGlutDisplay );
  GLUI_Master.set_glutReshapeFunc( myGlutReshape );  
  GLUI_Master.set_glutKeyboardFunc( myGlutKeyboard );
  GLUI_Master.set_glutSpecialFunc(SpecialKey);
  GLUI_Master.set_glutMouseFunc( myGlutMouse );
  glutMotionFunc( myGlutMotion );
  /* Register the idle callback with GLUI, *not* with GLUT */
  GLUI_Master.set_glutIdleFunc( myGlutIdle );
  /* Regular GLUT main loop */
  glutMainLoop();
  return EXIT_SUCCESS;
}
